﻿using System;
using System.Collections.Generic;
using System.IO;

namespace PpapCollage
{
    public class ExoReader : IDisposable
    {
        private readonly TextReader _reader;
        private bool _isStarted;
        private string _nextLine;

        public ExoReader(TextReader reader)
        {
            _reader = reader;
        }

        private void ReadLine()
        {
            _nextLine = _reader.ReadLine();
            if (_nextLine == "") this.ReadLine();
        }

        public ExoGroup Read()
        {
            if (!_isStarted)
            {
                this.ReadLine();
                _isStarted = true;
            }

            if (_nextLine == null) return null;

            if (_nextLine[0] != '[') throw new FormatException();

            var name = _nextLine.Substring(1, _nextLine.Length - 2);
            var props = new Dictionary<string, string>();

            while (true)
            {
                this.ReadLine();

                if (_nextLine == null || _nextLine.StartsWith('['))
                    break;

                var equalIndex = _nextLine.IndexOf('=');

                if (equalIndex < 0) throw new FormatException();

                props.Add(_nextLine.Remove(equalIndex), _nextLine.Substring(equalIndex + 1));
            }

            return new ExoGroup
            {
                Name = name,
                Properties = props
            };
        }

        public IReadOnlyList<ExoGroup> ReadAll()
        {
            var groups = new List<ExoGroup>();
            ExoGroup g;

            while ((g = this.Read()) != null)
                groups.Add(g);

            return groups;
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing) _reader.Dispose();
        }

        public void Dispose()
        {
            this.Dispose(true);
        }
    }
}
