﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;

namespace PpapCollage
{
    public class PenPineappleApplePen
    {
        public int Width { get; set; }
        public int Height { get; set; }
        public int Rate { get; set; } // FPS = Rate / Scale
        public int Scale { get; set; }

        public IReadOnlyList<ObjectAnimation> Pen1Animations { get; set; }
        public IReadOnlyList<ObjectAnimation> AppleAnimations { get; set; }
        public IReadOnlyList<ObjectAnimation> Pen2Animations { get; set; }
        public IReadOnlyList<ObjectAnimation> PineappleAnimations { get; set; }

        public static PenPineappleApplePen Load(IReadOnlyList<ExoGroup> groups)
        {
            var exeditProps = groups.First(x => x.Name == "exedit").Properties;

            return new PenPineappleApplePen()
            {
                Width = ParseInt(exeditProps["width"]),
                Height = ParseInt(exeditProps["height"]),
                Rate = ParseInt(exeditProps["rate"]),
                Scale = ParseInt(exeditProps["scale"]),
                Pen1Animations = LoadAnimation(groups, "ff0000"),
                AppleAnimations = LoadAnimation(groups, "0000ff"),
                Pen2Animations = LoadAnimation(groups, "00ff00"),
                PineappleAnimations = LoadAnimation(groups, "ffff00")
            };
        }

        private static int ParseInt(string s) =>
            int.Parse(s, NumberFormatInfo.InvariantInfo);

        private static float ParseFloat(string s) =>
            float.Parse(s, NumberFormatInfo.InvariantInfo);

        private static Tuple<float, float> ParseFloatPair(string s)
        {
            var split = s.Split(',');

            if (split.Length == 1)
            {
                var f = ParseFloat(s);
                return Tuple.Create(f, f);
            }

            // Length == 3 のときでも無視していくスタイル
            return Tuple.Create(ParseFloat(split[0]), ParseFloat(split[1]));
        }

        private static IReadOnlyList<ObjectAnimation> LoadAnimation(IReadOnlyList<ExoGroup> groups, string targetColor)
        {
            var count = groups.Count;
            var anims = new List<ObjectAnimation>();

            for (var i = 0; i < count; i++)
            {
                var g0 = groups[i].Properties;
                string color;
                if (!g0.TryGetValue("color", out color) || color != targetColor)
                    continue;

                var g1 = groups[i + 1].Properties;
                var x = ParseFloatPair(g1["X"]);
                var y = ParseFloatPair(g1["Y"]);
                var scale = ParseFloatPair(g1["拡大率"]);
                var rotate = ParseFloatPair(g1["回転"]);

                var g = groups[i - 1].Properties;

                anims.Add(new ObjectAnimation()
                {
                    StartFrame = ParseInt(g["start"]),
                    EndFrame = ParseInt(g["end"]),
                    StartPosition = new ObjectPosition()
                    {
                        X = x.Item1,
                        Y = y.Item1,
                        Scale = scale.Item1,
                        Rotate = rotate.Item1
                    },
                    EndPosition = new ObjectPosition()
                    {
                        X = x.Item2,
                        Y = y.Item2,
                        Scale = scale.Item2,
                        Rotate = rotate.Item2
                    }
                });
            }

            return anims;
        }
    }
}
